/*
 * Created on 2003. 11. 30.
 */
package gui;

import interfaces.Client;
import interfaces.PlatformFactory;
import interfaces.Plugin;
import interfaces.Server;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import addon.AddonPanel;

/**
 * @author hyoki
 * 
 */
public class BackGroundPanel extends JPanel {
	NamedTabbedJPanel server = null;
	NamedTabbedJPanel plugin = null;
	NamedTabbedJPanel client = null;
	RunningPanel runningPanel = null;
	AddonFactory addonFactory = null;
	Dimension defaultSize = new Dimension(400,300);
	private boolean updating;
	private PlatformFactory factory;
	
	/**
	 * @return Returns the clientSide.
	 */
	private NamedTabbedJPanel getClient() {
		if(client == null){
			final String type = "Client";
			client = new NamedTabbedJPanel(type,factory);
			setTableModel(type,client);
			client.setStateChangeListener(new ChangeListener(){
				public void stateChanged(ChangeEvent e) {
						if(((String)e.getSource()).equals("Refresh")){
							setTableModel(type,client);					
						}
				}});
			client.setPreferredSize(defaultSize);
		}
		return client;
	}

	private void setTableModel(String type,NamedTabbedJPanel panel) {
		panel.setTableModel(addonFactory.getModel(type));
	}

	/**
	 * @return Returns the pluginSide.
	 */
	private NamedTabbedJPanel getPlugin() {
		if(plugin == null){
			final String type ="Plugin";
			plugin = new NamedTabbedJPanel(type,factory);
			this.setTableModel(type,plugin);
			plugin.setStateChangeListener(new ChangeListener(){
				public void stateChanged(ChangeEvent e) {
					if(((String)e.getSource()).equals("Refresh")){
						setTableModel(type,client);
					}
				}});
			plugin.setPreferredSize(defaultSize);			
		}
		return plugin;
	}

	/**
	 * @return Returns the runPanel.
	 */
	private RunningPanel getRunningPanel() {
		if(runningPanel == null){
			runningPanel = new RunningPanel();
			runningPanel.setStartActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					if(isStartReady()){
						runningPanel.addHostes(getHostes());
					}
				}});			
			runningPanel.setPreferredSize(new Dimension(150,150));			
		}
		return runningPanel;
	}

	/**
	 * @return Returns the serverSide.
	 */
	private NamedTabbedJPanel getServer() {
		if(server == null){
			final String type = "Server";
			server = new NamedTabbedJPanel(type,factory);
			this.setTableModel(type,server);
			server.setStateChangeListener(new ChangeListener(){
				public void stateChanged(ChangeEvent e) {
					if(((String)e.getSource()).equals("Refresh")){
						setTableModel(type,client);					
					}
				}});
			server.setPreferredSize(defaultSize);
		}
		return server;
	}
	/**
	 * @param factory
	 * @param addonFactory2
	 */
	public BackGroundPanel(PlatformFactory factory, AddonFactory addonFactory) {
		this.factory = factory;
		this.addonFactory = addonFactory;		
		initComponent();
		update();
	}
	/**
	 * 
	 */
	private void update() {
		beginUpdating();
		updateStartButton();
		endUpdating();
	}
	/**
	 * 
	 */
	private void updateStartButton() {
		// TOTHINK do somthing.... T.T
//		getRunningPanel().getStartButton().setEnabled(isStartReady());
	}

	/**
	 * 
	 */
	private void beginUpdating() {
		updating = true;
	}

	/**
	 * 
	 */
	private void endUpdating() {
		updating = false;
	}

	/**
	 * 
	 */
	private boolean isUpdateing() {
		return updating;
	}

	/**
	 * 
	 */
	private void initComponent() {
		this.setLayout(new BorderLayout());
		this.setBorder(BorderFactory.createCompoundBorder(new EtchedBorder(EtchedBorder.RAISED,Color.lightGray,new Color(134, 134, 134)),BorderFactory.createEmptyBorder(5,5,5,5)));			
		
		this.add(getRunningPanel(),BorderLayout.SOUTH);
		this.add(getServer(),BorderLayout.WEST);
		this.add(getPlugin(),BorderLayout.CENTER);
		this.add(getClient(), BorderLayout.EAST);
		
		update();
	}
	/**
	 * @return
	 */
	public boolean isStartReady() {
		if(isPanelReady(getServer())&&isPanelReady(getClient())&&isPanelReady(getPlugin())){
			return true;
		}
		return false;
	}
	
	/**
	 * @param panel
	 * @return
	 */
	private boolean isPanelReady(NamedTabbedJPanel panel) {
		try {
			return panel.getSelectedPanel().isReady();			
		} catch (NullPointerException e) {
			return false;
		}
	}

	/**
	 * @return
	 */
	public Hostes getHostes() {
		if(isUpdateing()) return null;
		try{
		Server serverTemp = (Server) getHostFromSelectedPanel(getServer().getSelectedPanel());
		Client clientTemp = (Client) getHostFromSelectedPanel(getClient().getSelectedPanel());
		Plugin pluginTemp = (Plugin) getHostFromSelectedPanel(getPlugin().getSelectedPanel());
		return new Hostes(serverTemp,clientTemp,pluginTemp);		
		} catch (ClassCastException e) {
			return null;
		}
	}

	private Object getHostFromSelectedPanel(AddonPanel panel) {
		if( panel.isReady()){
		return panel.getHost();
		}
		return null;
	}
}
